<?php
/**
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * @package		svmods facebook connect for phpbb
 * @author      Vincent Miller <vsmiller[at]live[dot]com>
 * @license 	http://opensource.org/licenses/gpl-license.php GNU Public License
 * @link        http://svmods.com
 */



if (!defined('IN_PHPBB')){
	exit;
}

function svmods_facebook_avatar_url($uid)
{
	// this function has been changed to use the dynamic url 'trick'
	// I'm not fond of this at all... but phpbb allows dynamic urls for avatars,
	// so we'll go ahead and use it until they change it.
	// I'm leaving it as function as an easy way to update later if it does change.
	return 'http://graph.facebook.com/' . $uid . '/picture?.jpg';
}

function svmods_destroy_facebook_cookie()
{
	// nothing to explain here.
	// attempts to use phpbb native set_cookie, but follows up with standard setcookie.
	// not sure why they do it that way.
	global $config, $user;
	$temp=array($config['cookie_name'], $config['cookie_path'], $config['cookie_domain']);
	$config['cookie_name'] = 'fbs';
	$config['cookie_path'] = '/';
	$config['cookie_domain'] = true;
	$user->set_cookie($config['svmods_facebook_app_id'], NULL, time()-10);
	@setcookie('fbs_'.$config['svmods_facebook_app_id'], NULL, time()-10);
	$config['cookie_name'] = $temp[0];
	$config['cookie_path'] = $temp[1];
	$config['cookie_domain'] = $temp[2];
	$config['svmods_skip_fb_cookie'] = true;
}

function svmods_get_facebook_cookie()
{
	// nothing fancy, standard fb cookie parser
	// switched to used the beloved request_var (though it's not really
	// needed here).
	global $user, $config;
	// added this skip_fb_cookie condition in for destroy_cookie function above
	if (isset($config['svmods_skip_fb_cookie']))
	{
		return false;
	}
	$cookie = request_var('fbs_'.$config['svmods_facebook_app_id'], '', true, true);
	if (!empty($cookie))
	{
		if (!function_exists("htmlspecialchars_decode"))
		{
			// just in case we're running on really old php versions...
			function htmlspecialchars_decode($string, $quote_style = ENT_COMPAT)
			{
        		return strtr($string, array_flip(get_html_translation_table(HTML_SPECIALCHARS, $quote_style)));
			}
		}
		// we have to decode because request_var chewed up our cookie data.
		$cookie = htmlspecialchars_decode($cookie);
		$args = array();
		parse_str(trim($cookie, '\\"'), $args);
		ksort($args);
		$payload = '';
		foreach ($args as $key => $value)
		{
			if ($key!='sig')
			{
				$payload.=$key. '=' .$value;
			}
		}
		if (md5($payload.$config['svmods_facebook_secret']) === $args['sig'])
		{
			$user->data['svmods_facebook_data']['cookie'] = $args;
			$user->data['is_facebook_user'] = $args['uid'];
			$user->data['is_facebook_connected'] = 1;
			return true;
		}
	}
	return false;
}

function svmods_set_facebook_data($n=NULL, $id='me')
{
	// this is here because I intend to call it off a hook in future versions.
	// by default, simply fetches user data, but by passing the $n val
	// we generate urls for friends, etc.
	// we're creating 2 arrays that mark our data and what it is.
	global $user, $svmods_mcurl_urls, $svmods_set_returns;
	if (!is_array($svmods_mcurl_urls)){
		$svmods_mcurl_urls=array();
	}
	if (!is_array($svmods_set_returns)){
		$svmods_set_returns=array();
	}
	$svmods_mcurl_urls[] = 'https://graph.facebook.com/'.$id.'/'.$n.'?access_token='.$user->data['svmods_facebook_data']['cookie']['access_token'];
	if (empty($n)){
		$n='user';
	}
	$svmods_set_returns[] = $n;
}

function svmods_get_facebook_data($return=array())
{
	// our function to fetch the data from facebook.
	// will send it back as an array
	global $svmods_mcurl_urls, $svmods_set_returns, $phpbb_root_path, $phpEx;
	if (is_array($svmods_mcurl_urls))
	{
		$results = svmods_mcurl($svmods_mcurl_urls);
		if(!function_exists('json_decode'))
		{
			// this one's important
			// for php versions < 5.2.x we have to make a json_decode function
			include($phpbb_root_path.'includes/JSON.'.$phpEx);
			function json_decode($data)
			{
				$json=new Services_JSON();
				return($json->decode($data));
			}
		}
		for ($x=0, $s=sizeof($svmods_set_returns); $x<$s; $x++)
		{
			$return[$svmods_set_returns[$x]] = array();
			// the json encoded value is kept here for cache options in later versions
			$return[$svmods_set_returns[$x]]['json'] = $results[$x];
			$return[$svmods_set_returns[$x]]['decoded'] = json_decode($results[$x]);
		}
		// we have to adjust timezone in dst.
		// facebook returns a pre adjusted value!
		$dst=localtime(time(), true);
		$return['tm_isdst'] = $dst['tm_isdst'];
		if (isset($return['user']['decoded']->timezone) && ($dst['tm_isdst']))
		{
			$return['user']['decoded']->timezone--;
		}
	}
	return $return;
}

function svmods_create_fb_association($sql_ary)
{
	global $db;
	// stripped custom table, leaving array input for change later
	$db->sql_query('UPDATE '.USERS_TABLE ." SET user_svmods_fb_uid='".$sql_ary['user_svmods_fb_uid']."' WHERE user_svmods_fb_uid<1 AND user_id=".$sql_ary['user_id']);
}

function svmods_revoke_facebook_authorization($uid,$token)
{
	// simple function, sends off an old REST api call to fb to revoke application authorization
	global $db;
	$urls = array('https://api.facebook.com/method/auth.revokeAuthorization?uid='.$uid.'&access_token='.$token);
	svmods_mcurl($urls);
	// remove our account association
	$db->sql_query('UPDATE '.USERS_TABLE ." SET user_svmods_fb_uid=0 WHERE user_svmods_fb_uid=".$uid);
}

if (!function_exists('svmods_mcurl'))
{
	// this is my baby. love this function
	// not too important in initial version of hook, but when we add in friends data, 
	// like data, etc... curl_multi is beautiful.
	function svmods_mcurl($urls)
	{
		global $svmods_mcurl_urls;
		$results = array();
		// are we 5.2.x? Do we have curl to play with?
		if (function_exists('curl_init'))
		{
			$ch = array();
			for ($x=0, $s=sizeof($urls); $x<$s; $x++)
			{
				$ch[$x] = curl_init();
				// set some option, silence those that commonly throw warnings on vhosts, etc.
				curl_setopt($ch[$x], CURLOPT_URL, $urls[$x]);
				curl_setopt($ch[$x], CURLOPT_RETURNTRANSFER, true);
				curl_setopt($ch[$x], CURLOPT_HEADER, false);
				@curl_setopt($ch[$x], CURLOPT_SSL_VERIFYPEER, false);
				@curl_setopt($ch[$x], CURLOPT_FOLLOWLOCATION, true);
				@curl_setopt($ch[$x], CURLOPT_MAXREDIRS, 5);
			}
			$mh = curl_multi_init();
			for ($x=0, $s=sizeof($ch); $x<$s; $x++)
			{
				curl_multi_add_handle($mh, $ch[$x]);
			}
			$x=0; do
			{
				curl_multi_exec($mh, $x);
			} while($x>0);
	
			for ($x=0, $s=sizeof($ch); $x<$s; $x++)
			{
				$results[$x] = curl_multi_getcontent($ch[$x]);
			}
			for ($x=0, $s=sizeof($ch); $x<$s; $x++)
			{
				curl_multi_remove_handle($mh, $ch[$x]);
			}
			curl_multi_close($mh);
		}
		else
		{
			// no curl to play with :(
			// old school fgc
			for ($x=0, $s=sizeof($urls); $x<$s; $x++)
			{
				$results[$x] = @file_get_contents($urls[$x]);
			}
		}
		return $results;
	}
}

if (!function_exists('svmods_default_lang'))
{
	// we're just creating a function for throwing a default lang val if they
	// hadn't defined it in the lang files.
	function svmods_default_lang($key, $value)
	{
		global $user;
		if (!isset($user->lang[$key]))
		{
			$user->lang[$key] = $value;
		}
		return $user->lang[$key];
	}
}

if (!function_exists('svmods_set_unset_var'))
{
	// sorry, I'm not a fan of using isset everywhere in my conditions.
	// simple func to set an unset var
	function svmods_set_unset_var(&$var, $val=false)
	{
		if (!isset($var))
		{
  			$var=$val;
		}
	}
}

// debug mode cleanups
// using above function to set unset vars
svmods_set_unset_var($config['svmods_facebook_app_id']);
svmods_set_unset_var($config['svmods_facebook_secret']);
svmods_set_unset_var($config['svmods_facebook_compat'],0);
svmods_set_unset_var($config['svmods_facebook_use_remote_redirect'],'l');
svmods_set_unset_var($config['svmods_facebook_use_auto_assoc'],'e');
svmods_set_unset_var($config['svmods_facebook_use_lang'],'en_US');




// the template hook
function svmods_facebook_connect_template_hook(&$hook){
	global $db, $config, $user, $template, $phpEx, $phpbb_root_path, $cache;
	// stuff we only do for the ACP
	if (defined('IN_ADMIN'))
	{
		if ($config['svmods_facebook_compat']<1)
		{
			// 2 is compatible, 1 is not, 0 is untested.
			// if we're here, we need to fetch an error from fb.
			$temp = svmods_mcurl(array('https://graph.facebook.com/me/'));
			$check = json_decode($temp[0]);
			if (isset($check->error->type))
			{
				// we found what we're looking for
				unset($config['svmods_facebook_compat']);
				set_config('svmods_facebook_compat', 2, false);
			}
			else
			{
				// no good. shut it down.
				// user can run the included test file in the future to enable.
				// if they try to override by manually setting the config table value,
				// they're going to have nothing but problems.
				unset($config['svmods_facebook_compat']);
				set_config('svmods_facebook_compat', 1, false);
				$message = svmods_default_lang('SVMODS_FACEBOOK_COMPAT','There appears to be an issue with the svmods Facebook connect hook: the server has failed to retrieve data from Facebook, likely due to restrictions in the server settings. You may delete the hook file and purge the cache from the ACP.<br /><br />If you wish to reattempt installation later, you may run the test.php file that came with this hook, or you will need to delete the svmods_facebook_compat value from your config table and purge the cache again.');
				trigger_error($message);
			}
		}
		// lets just get out now if not compat.
		// we should have never been here.
		if ($config['svmods_facebook_compat'] != 2)
		{
			return NULL;
		}
		// handle our own custom ACP fields
		$submit = request_var('config', array('' => ''), true);
		if (isset($submit['svmods_facebook_app_id']))
		{
			$temp = $submit['svmods_facebook_app_id'];
			if ((!empty($temp)) && ($temp!=$config['svmods_facebook_app_id']))
			{
				unset($config['svmods_facebook_app_id']);
				set_config('svmods_facebook_app_id', $temp, false);
			}
			// moved inside for unchecked submit
			/* this is dead for now. left for future in case dynamic avatar urls ever change.
			svmods_set_unset_var($submit['svmods_facebook_use_remote_redirect'],'l');
			$temp = $submit['svmods_facebook_use_remote_redirect'];
			if ($temp!='r') $temp='l';
			if ($temp!=$config['svmods_facebook_use_remote_redirect'])
			{
				unset($config['svmods_facebook_use_remote_redirect']);
				set_config('svmods_facebook_use_remote_redirect', $temp, false);
			}
			*/
			// option to disable auto email association
			svmods_set_unset_var($submit['svmods_facebook_use_auto_assoc'],'e');
			$temp = $submit['svmods_facebook_use_auto_assoc'];
			$temp = ($temp != 'd') ? 'e' : 'd';
			if ($temp != $config['svmods_facebook_use_auto_assoc'])
			{
				unset($config['svmods_facebook_use_auto_assoc']);
				set_config('svmods_facebook_use_auto_assoc', $temp, false);
			}
		}
		if ((isset($submit['svmods_facebook_secret'])) && (!empty($submit['svmods_facebook_secret'])) && ($submit['svmods_facebook_secret'] != $config['svmods_facebook_secret']))
		{
			unset($config['svmods_facebook_secret']);
			set_config('svmods_facebook_secret', $submit['svmods_facebook_secret'], false);
		}
		if ((isset($submit['svmods_facebook_use_lang'])) && ($submit['svmods_facebook_use_lang'] != $config['svmods_facebook_use_lang']))
		{
			unset($config['svmods_facebook_use_lang']);
			set_config('svmods_facebook_use_lang', $submit['svmods_facebook_use_lang'], false);
		}
		if (!isset($user->data['user_svmods_fb_uid']))
		{
			// we don't appear to have been installed yet.
			// use db_tools to maintain compatibility with all phpbb supported dbs
			@include($phpbb_root_path.'includes/db/db_tools.'.$phpEx);
			$tools = new phpbb_db_tools($db);
			$tools->sql_column_add(USERS_TABLE, 'user_svmods_fb_uid', array('BINT',0));
		}
		// are we on the board settings screen?
		if ((isset($template->_tpldata['options'])) && (request_var('i', '')==='board') && (request_var('mode', '')==='settings'))
		{
			// loop through template to find our insert spot...
			for ($x=0, $s=sizeof($template->_tpldata['options']); $x<$s; $x++){
				if ((isset($template->_tpldata['options'][$x]['KEY'])) && ($template->_tpldata['options'][$x]['KEY']==='warnings_expire_days')){
					$t = $x-1;		// drop back & mark the spot
					$x = $s;		// kill off loop
				}
			}
			// $checkr = ($config['svmods_facebook_use_remote_redirect'] === 'r') ? 'checked' : NULL;
			$checka = ($config['svmods_facebook_use_auto_assoc'] === 'd') ? 'checked' : NULL;
			$temp = array_chunk($template->_tpldata['options'], $t, false);
			$temp[0][]=array(
						'S_LEGEND'	=> 1,
						'LEGEND'	=> 'SVmods Facebook Settings',
						'S_ROW_COUNT' => $t++
					);
			$temp[0][]=array(
						'KEY'	=> 'svmods_facebook_app_id',
						'TITLE' => 'Facebook application ID',
						'S_EXPLAIN'	=> 1,
						'TITLE_EXPLAIN' => 'Your registered Facebook application ID.',
						'CONTENT'	=> '<input id="svmods_facebook_app_id" type="text" size="20" name="config[svmods_facebook_app_id]" value="'.$config['svmods_facebook_app_id'].'" />',
						'S_ROW_COUNT' => $t++
					);
			$temp[0][]=array(
						'KEY'	=> 'svmods_facebook_secret',
						'TITLE' => 'Facebook application secret',
						'S_EXPLAIN'	=> 1,
						'TITLE_EXPLAIN' => 'Your registered Facebook application secret.',
						'CONTENT'	=> '<input id="svmods_facebook_secret" type="text" size="20" name="config[svmods_facebook_secret]" value="'.$config['svmods_facebook_secret'].'" />',
						'S_ROW_COUNT' => $t++
					);
			$temp[0][]=array(
						'KEY'	=> 'svmods_facebook_use_lang',
						'TITLE' => 'Facebook language setting',
						'S_EXPLAIN'	=> 1,
						'TITLE_EXPLAIN' => 'For english (or if you\'re unsure about this setting) leave set to en_US. For more information and valid options, visit <a href="http://svmods.com/facebook-connect-for-phpbb.html" target="_blank">SVmods</a>.',
						'CONTENT'	=> '<input id="svmods_facebook_use_lang" type="text" size="6" name="config[svmods_facebook_use_lang]" value="'.$config['svmods_facebook_use_lang'].'" />',
						'S_ROW_COUNT' => $t++
					);
			$temp[0][]=array(
						'KEY'	=> 'svmods_facebook_use_auto_assoc',
						'TITLE' => 'Disable automatic association by email',
						'S_EXPLAIN'	=> 1,
						'TITLE_EXPLAIN' => 'Whenever a new Facebook user connects, by default the mod will automatically associate it to an existing account (if an account with the same email address is found and email re-use is disabled). Check here to disable this automatic association.',
						'CONTENT'	=> '<input id="svmods_facebook_use_auto_assoc" type="checkbox" name="config[svmods_facebook_use_auto_assoc]" value="d" '.$checka.' />',
						'S_ROW_COUNT' => $t++
					);
			/* removed while using dynamic avatar urls... left for change.
			$temp[0][]=array(
						'KEY'	=> 'svmods_facebook_use_remote_redirect',
						'TITLE' => 'Use remote redirect for Facebook avatars',
						'S_EXPLAIN'	=> 1,
						'TITLE_EXPLAIN' => 'Check here to use the svmods remote redirect script for users Facebook images as avatars. It\'s recommended that you implement your own htaccess redirect. For more information visit <a href="http://svmods.com/facebook-connect-for-phpbb.html" target="_blank">SVmods</a>.',
						'CONTENT'	=> '<input id="svmods_facebook_use_remote_redirect" type="checkbox" name="config[svmods_facebook_use_remote_redirect]" value="r" '.$checkr.' />',
						'S_ROW_COUNT' => $t++
					);
			*/
			// bring them back together...
			$template->_tpldata['options'] = array_merge($temp[0], $temp[1]);
			// fix the row count values
			for ($x=$t, $s=sizeof($template->_tpldata['options']); $x<$s; $x++)
			{
				$template->_tpldata['options'][$x]['S_ROW_COUNT']=$t++;
			}
		}
	}
	// out of admin water.
	// here's for the everyday use...
	if ((!empty($config['svmods_facebook_app_id'])) && (!empty($config['svmods_facebook_secret'])) && ($config['svmods_facebook_compat']==2))
	{
		$template->assign_var('SVMODS_FACEBOOK_APP_ID', $config['svmods_facebook_app_id']);
		$template->assign_var('SVMODS_FACEBOOK_XMLNS', ' xmlns:og="http://opengraphprotocol.org/schema/"
 xmlns:fb="http://www.facebook.com/2008/fbml" ');
		svmods_set_unset_var($template->_rootref['TRANSLATION_INFO']);
		$template->assign_var('TRANSLATION_INFO', '<a href="http://svmods.com/facebook-connect-for-phpbb.html" title="Facebook Connect Login and Registration for phpBB">Facebook connect for phpBB</a> by SVmods.<br />'.$template->_rootref['TRANSLATION_INFO']);
		$template->assign_var('SVMODS_FACEBOOK_UID', $user->data['user_svmods_fb_uid']);
		$url = str_replace('mode=logout', 'mode=login', $user->page['page']);
		$delim = (strpos($url,'?') === false) ? '?' : '&';
		// this sets up our fix for bad browsers... 
		$url = (strpos($url, 'svmods_check_cookie')) ? $url.$delim.'svmods_check_cookie=fb' : $url;
		/* the original footer js, now replaced with async in overall_header
		$template->assign_var('SVMODS_FACEBOOK_JS', '<div id="fb-root"></div><script src="http://connect.facebook.net/'.$config['svmods_facebook_use_lang'].'/all.js"></script><script>FB.init({appId: \''.$config['svmods_facebook_app_id'].'\', status: false, cookie: true, xfbml: true}); FB.Event.subscribe(\'auth.login\', function(response){window.location.replace(\''.$url.'\');});</script>'); */
		$template->assign_var('SVMODS_FACEBOOK_JS', '<div id="fb-root"></div><script type="text/javascript">	  window.fbAsyncInit = function() {FB.init({appId: \''.$config['svmods_facebook_app_id'].'\', status: false, cookie: true, xfbml: true});  FB.Event.subscribe(\'auth.login\', function() {window.location.replace(\''.$url.'\');});};(function() {var e = document.createElement(\'script\'); e.async = true; e.src = document.location.protocol + \'//connect.facebook.net/'.$config['svmods_facebook_use_lang'].'/all.js\'; document.getElementById(\'fb-root\').appendChild(e);}());</script>');
		$template->assign_var('SVMODS_FACEBOOK_INT',svmods_default_lang('SVMODS_FACEBOOK_INT', 'Facebook Integration'));
		$template->assign_var('SVMODS_FACEBOOK_INT_EXPLAIN',svmods_default_lang('SVMODS_FACEBOOK_INT_EXPLAIN', 'Grant/revoke Facebook application authorization.'));
		$template->assign_var('SVMODS_FACEBOOK_REVOKE',svmods_default_lang('SVMODS_FACEBOOK_REVOKE', 'Revoke Authorization'));
		if (($user->data['user_svmods_fb_uid']) && ($config['allow_avatar_remote']) && (request_var('mode', '')==='avatar'))
		{
			$template->assign_var('SVMODS_USE_FB_PIC', svmods_default_lang('SVMODS_USE_FB_PIC', 'Use Facebook image'));
			$template->assign_var('SVMODS_USE_FB_PIC_EXPLAIN', svmods_default_lang('SVMODS_USE_FB_PIC_EXPLAIN', 'Check to use your Facebook profile image as your avatar.'));
		}
		// do we need buttons?
		// no buttons for admin login.
		if ((!defined('ADMIN_START')) && ($user->data['user_svmods_fb_uid']<1))
		{
			// permissions string. requesting more than we use in basic connect, but will be used later.
			$perms = 'perms="email, user_birthday, user_interests, user_likes, user_location, user_website, publish_stream"';
			if ($user->data['user_id'] != ANONYMOUS)
			{
				// this is user, but not fb associated.
				$template->assign_var('SVMODS_FACEBOOK_LOGIN_BUTTON', '<fb:login-button size="medium" '.$perms.'>'.svmods_default_lang('SVMODS_FACEBOOK_CONNECT_BUTTON_TEXT', 'Connect your Facebook account').'</fb:login-button>');
			}
			else
			{
				$template->assign_var('SVMODS_FACEBOOK_LOGIN_BUTTON', '<fb:login-button size="medium" '.$perms.'>'.svmods_default_lang('SVMODS_FACEBOOK_LOGIN_BUTTON_TEXT', 'Login with Facebook!').'</fb:login-button>');
				$template->assign_var('SVMODS_FACEBOOK_REGISTER_BUTTON', '<fb:login-button size="large" '.$perms.'>'.svmods_default_lang('SVMODS_FACEBOOK_REGISTER_BUTTON_TEXT', 'Register with your Facebook account!').'</fb:login-button>');
			}
		}
	}
}

// the user hook.
function svmods_facebook_connect_user_hook(&$hook)
{
	global $config, $cache, $user, $auth, $db, $template, $phpbb_root_path, $phpEx;
	svmods_set_unset_var($user->data['is_facebook_user'], 0);
	// not for bots, and not when the board is disabled.
	if (($user->data['is_bot']) || ($config['board_disable']))
	{
		// throw app id out to kill template hook
		$config['svmods_facebook_app_id']=NULL;
		return NULL;
	}
	if ((!empty($config['svmods_facebook_app_id'])) && (!empty($config['svmods_facebook_secret'])) && ($config['svmods_facebook_compat']==2))
	{
		if ((request_var('mode', '') === 'avatar') && (request_var('svmods_use_fb_pic', false)))
		{
			set_var($_REQUEST['remotelink'],svmods_facebook_avatar_url($user->data['user_svmods_fb_uid']),'string');
			set_var($_REQUEST['width'],50,'integer');
			set_var($_REQUEST['height'],50,'integer');
		}
		else if (request_var('mode','') === 'logout')
		{
			// we destroy the cookie so we don't autologin
			svmods_destroy_facebook_cookie();
			$temp = false;
		}
		else
		{
			$temp = svmods_get_facebook_cookie();
		}
		if (request_var('svmods_fb_cancel', false))
		{
			// hit cancel on link/create screen. revoke and kill cookie.
			svmods_revoke_facebook_authorization($user->data['svmods_facebook_data']['cookie']['uid'], $user->data['svmods_facebook_data']['cookie']['access_token']);
			svmods_destroy_facebook_cookie();
			// this is for browsers that autocompleted our login form...
			set_var($_REQUEST['password'], NULL, 'NULL');
			$temp = false;
		}
		else if ((request_var('svmods_check_cookie','')==='fb') && (!$temp))
		{
			// this is our bad browser check.
			// destroy a bad cookie
			svmods_destroy_facebook_cookie();
			$message=svmods_default_lang('SVMODS_FB_LOGIN_COOKIE_ERROR', 'An error appears to have occurred with your browser. If you wish to use the sites Facebook logins or registration features, please update/repair your browser, or use a different browser.<br /><br />We applogize for the inconvenience.').'<br /><br />'.sprintf($user->lang['RETURN_INDEX'], '<a href="'.append_sid("{$phpbb_root_path}index.$phpEx").'">', '</a>');
			trigger_error($message);
		}
		if ($user->data['user_svmods_fb_uid']>0)
		{
			// we're already associated with an fb account
			if (request_var('svmods_revoke_fb', false))
			{
				svmods_destroy_facebook_cookie();
				// clicked revoke auth button, give it a try if an access token is available.
				if (isset($user->data['svmods_facebook_data']['cookie']['access_token']))
				{
					svmods_revoke_facebook_authorization($user->data['svmods_facebook_data']['cookie']['uid'],$user->data['svmods_facebook_data']['cookie']['access_token']);
					$message = svmods_default_lang('SVMODS_FACEBOOK_REVOKED', 'The request to revoke application authorization was sent to Facebook. You can check your application settings within Facebook itself to ensure it was successful.');
				}
				else
				{
					// no token avail. just tell em to go to their fb app settings.
					$message = svmods_default_lang('SVMODS_FACEBOOK_NOT_REVOKED', 'Your browser does not contain a current access token to revoke authorization. You may do so on Facebook in the application settings of the account menu.');
				}
				$redirect = str_replace('svmods_check_cookie=fb', '', $user->page['page']);
				$l_redirect=(($redirect === "index.$phpEx") ? $user->lang['RETURN_INDEX'] : $user->lang['RETURN_PAGE']);
				trigger_error($message . '<br /><br />' . sprintf($l_redirect, '<a href="' . $redirect . '">', '</a>'));
			}
			else if ($user->data['user_svmods_fb_uid']===$user->data['is_facebook_user'])
			{
				// nothing to do here. already associated and logged in
				return NULL;
			}
		}
		if ($temp){
			// at this point, we have an fb uid, but not logged into an associated phpbb account
			// we figure out what we need to do with it.
			$sql = 'SELECT user_id FROM ' . USERS_TABLE . ' WHERE user_svmods_fb_uid=' . $user->data['is_facebook_user'];
			$result = $db->sql_query($sql);
			$user_id = (int) $db->sql_fetchfield('user_id');
			$db->sql_freeresult($result);
			if ($user->data['user_id'] != ANONYMOUS)
			{
				if (($user_id<1) && ($user->data['user_svmods_fb_uid']<1))
				{
					// we are logged into phpbb, not associated, and have a new fb uid to work with.
					// create association
					$sql_ary = array('user_id'=>$user->data['user_id'], 'user_svmods_fb_uid'=>$user->data['is_facebook_user']);
					svmods_create_fb_association($sql_ary);
					$message=svmods_default_lang('SVMODS_FACEBOOK_NOW_ASSOCIATED', 'Your Facebook account is now associated with your '.$config['sitename'].' account.');
					$redirect = str_replace('svmods_check_cookie=fb', '', $user->page['page']);
					$l_redirect = (($redirect === "index.$phpEx") ? $user->lang['RETURN_INDEX'] : $user->lang['RETURN_PAGE']);
					$redirect = meta_refresh(3, $redirect);
					trigger_error($message . '<br /><br />' . sprintf($l_redirect, '<a href="' . $redirect . '">', '</a>'));
				}
				else if ($user_id != $user->data['user_id'])
				{
					// mismatch. we know of an account to this fb uid, but not the current one!
					// the phpbb account we are logged in with belongs to another fb account...
					if ($user_id<1){
						// do we have an associated account for this fb uid? if not, might as well revoke auth.
						svmods_revoke_facebook_authorization($user->data['svmods_facebook_data']['cookie']['uid'], $user->data['svmods_facebook_data']['cookie']['access_token']);
					}
					$message = svmods_default_lang('SVMODS_FACEBOOK_MISMATCH','The '.$config['sitename'].' account you are logged in with is associated with a different Facebook account. Please use that Facebook account when logging in.');
					svmods_destroy_facebook_cookie();
					$redirect = str_replace('svmods_check_cookie=fb', '', $user->page['page']);
					$l_redirect = (($redirect === "index.$phpEx") ? $user->lang['RETURN_INDEX'] : $user->lang['RETURN_PAGE']);
					$redirect = meta_refresh(3, $redirect);
					trigger_error($message . '<br /><br />' . sprintf($l_redirect, '<a href="' . $redirect . '">', '</a>'));
				}
			}
			else
			{
				$mode=request_var('mode', '');
				$password=request_var('password', '', true);
				if ($user_id)
				{
					// lets log them in.
					$user->session_create($user_id, false, true);
					$auth->acl($user->data);
					$user->setup();
					// repopulate our cookie data
					svmods_get_facebook_cookie();
				}
				else if (($mode!='terms') && ($mode!='privacy'))
				{
					// we don't interfere with calls to view site terms and privacy
					if (!empty($password)){
						// pass submitted, assuming attempt to link.
						// we fake non compat to stop from displayinf connect button and js on the login page
						// login page should only actually display if they failed the login when attempting to link account.
						$config['svmods_facebook_app_compat']=1;
						$user->add_lang('ucp');
						login_box();
					}
					// at this point, we have a new fb uid and need to get their fb user data
					svmods_set_facebook_data();
					$user->data['svmods_facebook_data'] = svmods_get_facebook_data($user->data['svmods_facebook_data']);
					$email = strtolower(trim($user->data['svmods_facebook_data']['user']['decoded']->email));
					if (($config['allow_emailreuse']) || ($config['svmods_facebook_use_auto_assoc']==='d'))
					{
						// this disables the automatic account association by email match.
						$user_id = false;
					}
					else
					{
						// look for an account matching email addres and not already associated with another fb uid
						$sql = 'SELECT user_id FROM '.USERS_TABLE.' WHERE user_email=\''.$email.'\' AND user_svmods_fb_uid<1';
						$result = $db->sql_query($sql);
						$user_id = (int) $db->sql_fetchfield('user_id');
						$db->sql_freeresult($result);
					}
					if (($user_id) && ($user_id!=ANONYMOUS))
					{
						// we found one, create the association and log them in.
						$sql_ary = array('user_id'=>$user_id, 'user_svmods_fb_uid'=>$user->data['is_facebook_user']);
						svmods_create_fb_association($sql_ary);
						$user->session_create($user_id, false, true);
						$auth->acl($user->data);
						$user->setup();
						// repopulate cookie data
						svmods_get_facebook_cookie();
						$message = svmods_default_lang('SVMODS_FACEBOOK_NOW_ASSOCIATED', 'Your Facebook account is now associated with your '.$config['sitename'].' account.');
						$redirect = str_replace('svmods_check_cookie=fb', '', $user->page['page']);
						$l_redirect = (($redirect === "index.$phpEx") ? $user->lang['RETURN_INDEX'] : $user->lang['RETURN_PAGE']);
						$redirect = meta_refresh(3, $redirect);
						trigger_error($message . '<br /><br />' . sprintf($l_redirect, '<a href="' . $redirect . '">', '</a>'));
						
					}
					else
					{
						// looks like we're dealing with a completely unknown fb uid.
						$user->add_lang('ucp');
						if ($config['check_dnsbl'])
						{
							if (($dnsbl = $user->check_dnsbl('register')) !== false){
								trigger_error(sprintf($user->lang['IP_BLACKLISTED'], $user->ip, $dnsbl[1]));
							}
						}
						if (!function_exists('user_get_id_name'))
						{
							include($phpbb_root_path . 'includes/functions_user.' . $phpEx);
						}
						// do we have a submitted username?
						$svmods_username = utf8_normalize_nfc(request_var('svmods_username', '', true));
						if (!empty($svmods_username))
						{
							$invalid_username = validate_username($svmods_username);
							// at this point, if username submitted is not valid, invalud_username carries our error message.
							if (!$invalid_username)
							{
								$coppa = request_var('coppa', false);
								$group_name = ($coppa) ? 'REGISTERED_COPPA' : 'REGISTERED';				
								$sql = 'SELECT group_id FROM ' . GROUPS_TABLE . ' WHERE group_name = \'' . $db->sql_escape($group_name) . '\' AND group_type = ' . GROUP_SPECIAL;
								$result = $db->sql_query($sql);
								$row = $db->sql_fetchrow($result);
								$db->sql_freeresult($result);				
								if (!$row)
								{
									trigger_error('NO_GROUP');
								}				
								$group_id = $row['group_id'];
								// we removed user activation option for the fb users
								if (($coppa || $config['require_activation'] == USER_ACTIVATION_ADMIN) && $config['email_enable'])
								{
									$user_actkey = gen_rand_string(10);
									$key_len = 54 - (strlen($server_url));
									$key_len = ($key_len < 6) ? 6 : $key_len;
									$user_actkey = substr($user_actkey, 0, $key_len);				
									$user_type = USER_INACTIVE;
									$user_inactive_reason = INACTIVE_REGISTER;
									$user_inactive_time = time();
								}
								else
								{
									$user_type = USER_NORMAL;
									$user_actkey = '';
									$user_inactive_reason = 0;
									$user_inactive_time = 0;
								}
								// create a random password for fb registered accounts
								$rand_pass = gen_rand_string(8);
								$server_url = generate_board_url();
								$user_row = array(
									'username'				=> $svmods_username,
									'user_password'			=> phpbb_hash($rand_pass),
									'user_email'			=> $email,
									'group_id'				=> (int) $group_id,
									'user_timezone'			=> (float) $user->data['svmods_facebook_data']['user']['decoded']->timezone,
									'user_dst'				=> $user->data['svmods_facebook_data']['tm_isdst'],
									'user_lang'				=> $config['default_lang'],
									'user_type'				=> $user_type,
									'user_actkey'			=> $user_actkey,
									'user_ip'				=> $user->ip,
									'user_regdate'			=> time(),
									'user_inactive_reason'	=> $user_inactive_reason,
									'user_inactive_time'	=> $user_inactive_time,
									'user_svmods_fb_uid'	=> $user->data['is_facebook_user']
								);
								if (($config['allow_avatar_remote']) && (request_var('svmods_use_fb_pic', 0)>0))
								{
									// this only works with jpg fb images. if test account or blank will null
									$avatar = array('remotelink'=>svmods_facebook_avatar_url($user->data['is_facebook_user']), 'width'=>50, 'height'=>50);
									$error = array();
									list($user_row['user_avatar_type'], $user_row['user_avatar'], $user_row['user_avatar_width'], $user_row['user_avatar_height']) = avatar_remote($avatar, $error);
								}
								if ($config['new_member_post_limit'])
								{
									$user_row['user_new'] = 1;
								}
								// register the new user
								$user_id = user_add($user_row);				
								if ($user_id===false)
								{
									trigger_error('NO_USER', E_USER_ERROR);
								}
								else
								{
									// all seems to have gone well.
									// create our association with $user_id
									$sql_ary = array('user_id'=>$user_id, 'user_svmods_fb_uid'=>$user->data['is_facebook_user']);
									svmods_create_fb_association($sql_ary);
									$user->data['is_facebook_user']=1;
									$user->data['is_facebook_connected']=1;
									// all the email stuff...
									if ($coppa && $config['email_enable'])
									{
										$message = $user->lang['ACCOUNT_COPPA'];
										$email_template = 'coppa_welcome_inactive';
										if (file_exists($user->lang_path.$user->lang_name.'/email/svmods_fb_coppa_welcome_inactive.txt'))
										{
											$email_template = 'svmods_fb_coppa_welcome_inactive';
										}
									}
									else if ($config['require_activation'] == USER_ACTIVATION_ADMIN && $config['email_enable'])
									{
										$message = $user->lang['ACCOUNT_INACTIVE_ADMIN'];
										$email_template = 'admin_welcome_inactive';
										if (file_exists($user->lang_path.$user->lang_name.'/email/svmods_fb_admin_welcome_inactive.txt'))
										{
											$email_template = 'svmods_fb_admin_welcome_inactive';
										}
									}
									else
									{
										$message = svmods_default_lang('SVMODS_FACEBOOK_ACCOUNT_ADDED', 'Thank you for registering, your account has been created. You may now login using your Facebook account.<br /><br />A random password has been created for your account. Should you wish to use the standard login in the future, you can reset your password through the password recovery link on the login page.');
										$email_template = 'user_welcome';
										if (file_exists($user->lang_path.$user->lang_name.'/email/svmods_fb_user_welcome.txt'))
										{
											$email_template = 'svmods_fb_user_welcome';
										}
									}
									if ($config['email_enable'])
									{
										include($phpbb_root_path.'includes/functions_messenger.'.$phpEx);					
										$messenger = new messenger(false);					
										$messenger->template($email_template, $config['default_lang']);					
										$messenger->to($email, $svmods_username);					
										$messenger->headers('X-AntiAbuse: Board servername - ' . $config['server_name']);
										$messenger->headers('X-AntiAbuse: User_id - ' . $user_id);
										$messenger->headers('X-AntiAbuse: Username - ' . $svmods_username);
										$messenger->headers('X-AntiAbuse: User IP - ' . $user->ip);					
										$messenger->assign_vars(array(
											'WELCOME_MSG'	=> htmlspecialchars_decode(sprintf($user->lang['WELCOME_SUBJECT'], $config['sitename'])),
											'USERNAME'		=> htmlspecialchars_decode($svmods_username),
											'PASSWORD'		=> htmlspecialchars_decode($rand_pass),
											'U_ACTIVATE'	=> "$server_url/ucp.$phpEx?mode=activate&u=$user_id&k=$user_actkey")
										);
										if ($coppa)
										{
											$messenger->assign_vars(array(
												'FAX_INFO'		=> $config['coppa_fax'],
												'MAIL_INFO'		=> $config['coppa_mail'],
												'EMAIL_ADDRESS'	=> $email)
											);
										}
										$messenger->send(NOTIFY_EMAIL);					
										if ($config['require_activation'] == USER_ACTIVATION_ADMIN)
										{
											// Grab an array of user_id's with a_user permissions ... these users can activate a user
											$admin_ary = $auth->acl_get_list(false, 'a_user', false);
											$admin_ary = (!empty($admin_ary[0]['a_user'])) ? $admin_ary[0]['a_user'] : array();					
											$where_sql = ' WHERE user_type = ' . USER_FOUNDER;					
											if (sizeof($admin_ary))
											{
												$where_sql .= ' OR ' . $db->sql_in_set('user_id', $admin_ary);
											}					
											$sql='SELECT user_id, username, user_email, user_lang, user_jabber, user_notify_type FROM '.USERS_TABLE.' '.$where_sql;
											$result = $db->sql_query($sql);					
											while ($row = $db->sql_fetchrow($result))
											{
												$messenger->template('admin_activate', $row['user_lang']);
												$messenger->to($row['user_email'], $row['username']);
												$messenger->im($row['user_jabber'], $row['username']);					
												$messenger->assign_vars(array(
													'USERNAME'			=> htmlspecialchars_decode($svmods_username),
													'U_USER_DETAILS'	=> "$server_url/memberlist.$phpEx?mode=viewprofile&u=$user_id",
													'U_ACTIVATE'		=> "$server_url/ucp.$phpEx?mode=activate&u=$user_id&k=$user_actkey")
												);					
												$messenger->send($row['user_notify_type']);
											}
											$db->sql_freeresult($result);
										}
									}
									$message.='<br /><br />'.sprintf($user->lang['RETURN_INDEX'], '<a href="'.append_sid("{$phpbb_root_path}index.$phpEx").'">', '</a>');
									trigger_error($message);
								}
							}
							else
							{
								$svmods_username = NULL;
							}
						}
						// trigger error screen and present both login and register options!
						// overwrite the generic information title
						$user->lang['INFORMATION'] = svmods_default_lang('SVMODS_FACEBOOK_REGISTER_PAGE_TITLE', 'Welcome to '.$config['sitename'].', '.$user->data['svmods_facebook_data']['user']['decoded']->first_name.'! Create/link your account with Facebook');						
						$username_explain = sprintf($user->lang[$config['allow_name_chars'] . '_EXPLAIN'], $config['min_name_chars'], $config['max_name_chars']);
						$svmods_username = trim(utf8_normalize_nfc($user->data['svmods_facebook_data']['user']['decoded']->name));
						if (validate_username($svmods_username))
						{
							// fb name isn't going to fly
							// let's remove spaces.
							$svmods_username = str_replace(' ', '', $svmods_username);
							if (validate_username($svmods_username))
							{
								// that's no good either. 
								// just erase it. we're not doing this all day.
								$svmods_username = NULL;
							}
						}
						// make our message for trigger error.
						$message = NULL;
						if (!empty($invalid_username))
						{
							$message = '<div class="error">' . $user->lang[$invalid_username.'_USERNAME'] . '</div>';
						}
						else if (!empty($svmods_fb_link['error_msg']))
						{
							$message = '<div class="error">' . $user->lang[$svmods_fb_link['error_msg']] . '</div>';
						}
						// just creating the 2 forms for linking to existing accounts or creating new one.
						$message.='<form method="post" class="headerspace">
							<h3>'.svmods_default_lang('SVMODS_FACEBOOK_NEW_CONNECT_TEXT', 'Existing users: login to link your Facebook account to your '.$config['sitename'].' account.').'</h3>
							<fieldset class="quick-login">
							<label for="username">'.$user->lang['USERNAME'].':</label>&nbsp;<input type="text" name="username" id="username" size="10" class="inputbox" title="'.$user->lang['USERNAME'].'" />
							<label for="password">'.$user->lang['PASSWORD'].':</label>&nbsp;<input type="password" name="password" id="password" size="10" class="inputbox" title="'.$user->lang['PASSWORD'].'" />
							<input type="submit" name="login" value="'.$user->lang['LOGIN'].'" class="button2" />&nbsp;
							<input type="submit" name="svmods_fb_cancel" value="'.$user->lang['CANCEL'].'" class="button2" />
							</fieldset>
							</form><br />';
						$message.='<form method="post" class="headerspace">
							<h3>'.svmods_default_lang('SVMODS_FACEBOOK_NEW_REGISTER_TEXT', 'New to '.$config['sitename'].'? Register your username.').'</h3>
							'.$user->lang['LOGIN_INFO'].'<br /><br />'.svmods_default_lang('SVMODS_FACEBOOK_NEW_REGISTER_TEXT', 'By submitting your registration, you agree to the site <a href="'.append_sid('ucp.'.$phpEx.'?mode=terms').'">terms of use</a> and <a href="'.append_sid('ucp.'.$phpEx.'?mode=privacy').'">privacy policy</a>.').'<br />'.sprintf($user->lang[$config['allow_name_chars'] . '_EXPLAIN'], $config['min_name_chars'], $config['max_name_chars']).'<br /><br />
							<fieldset class="quick-login">
							<label for="username">'.$user->lang['USERNAME'].':</label>&nbsp;<input type="text" name="svmods_username" id="svmods_username" size="10" class="inputbox" title="'.$user->lang['USERNAME'].'" value="'.$svmods_username.'" />&nbsp;<input type="submit" name="register" value="'.$user->lang['REGISTER'].'" class="button2" />&nbsp;
							<input type="submit" name="svmods_fb_cancel" value="'.$user->lang['CANCEL'].'" class="button2" />';
						if ($config['allow_avatar_remote']){
							$message.='&nbsp;&nbsp;&nbsp;<input type="checkbox" id="svmods_use_fb_pic" name="svmods_use_fb_pic" value="1" checked />&nbsp;'.svmods_default_lang('SVMODS_USE_FB_PIC', 'Use Facebook image');
						}
						$message.='</fieldset></form><br /><br />';
						trigger_error($message);
					}
				}
			}
		}
	}
}



// only register the hooks if compatible, not diabled board, and courtesy val for other modders isn't defined.
// other mod authors may define NO_SVMODS_FACEBOOK_CONNECT in their custom pages if they wish to avoid the
// interuption: when this hook sees an unknown fb uid it interupts behind user->setup.
// example situation: a user during beta came forward with an im/chat mod that requests a custom script
// with a json output. The hook saw the cookie after the user clicked an fb login button, and the other
// mod faulted because this hook threw out a trigger_error screen instead of json.
// so, the courtesy value they can define. NO_SVMODS_FACEBOOK_CONNECT
if ((!$config['board_disable']) && ($config['svmods_facebook_compat']!=1) && (!defined('NO_SVMODS_FACEBOOK_CONNECT')))
{
	$phpbb_hook->register('phpbb_user_session_handler', 'svmods_facebook_connect_user_hook');
	$phpbb_hook->register(array('template','display'), 'svmods_facebook_connect_template_hook');
}



